import dotenv from 'dotenv';
import { Bus, FisAppActor, FisMessage } from "../interface/transport.interface";
import { ConnectionAdapter } from "./connector.base";
import { AdapterProfile, AdaptorTransmissionRole, ConnectionState, ReceiverConnectionAdapter as ReceiverConnectionAdapterInterface, TransportEvent, TransportMessage, TransportService } from "../interface/connector.interface";
import { BehaviorSubject, filter, map, Observable, Observer, Subject, Subscription, takeWhile } from 'rxjs';
import { v4 as uuidv4 } from 'uuid'

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class ReceiverConnectionAdapter extends ConnectionAdapter implements ReceiverConnectionAdapterInterface {
    event!: Subject<TransportEvent>

    constructor(adapter: TransportService, adapterId: string, event: Subject<TransportEvent>) {
        super()
        // logic here
        this.event = event
        this.adaptorTransmissionRole = AdaptorTransmissionRole.Receiver
        this.connector = adapter

        this.setAdapterProfile(adapterId)
    }


    setAdapterProfile(adapterId: string): void {
        this.connectorProfile = {
            id: adapterId,
            connectionState: this.connectionStateBus,
            adapter: this.connector
        }
    }

    getMessageBus(bus: Bus): Observable<any> {
        return new Observable((observable: Observer<any>) => {
            if (bus == Bus.GeneralBus) {
                const subscription: Subscription = this.event.pipe(
                    filter((message: TransportEvent) => message.event === 'New Message'),
                    // take message only specific for this adapter. Although that itself wouldn't be necessary, considerng everything goes through transportEvent. I guess it's for better management
                    filter((message: TransportEvent) => (message.data as TransportMessage).target == this.connectorProfile.id),
                    takeWhile((message: TransportEvent) => {
                        const shouldTake = ((message.data as TransportMessage).payload as FisMessage).data !== 'Complete';
                        if (!shouldTake) {
                            observable.complete();  // Ensure the observer is completed
                        }
                        return shouldTake;
                    }),
                    map(message => message.data as TransportMessage)
                ).subscribe((message: TransportMessage) => {
                    observable.next(message);
                });

                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
            if (bus == Bus.ResponseMessageBus) {
                /// logic here
            }
            if (bus == Bus.NotificationMessageBus) {
                /// logic here
            }
            if (bus == Bus.ErrorMessageBus) {
                /// logic here
            }
        });
    }

    getInfo(): AdapterProfile {
        return this.connectorProfile
    }

    subscribeMessages(messageFilter: any): Observable<any> {
        throw new Error('Method not implemented.');
    }
}


