import dotenv from 'dotenv';
import { Bus, FisAppActor, FisMessage, TransmissionMessage } from "../interface/transport.interface";
import { ConnectionAdapter } from "./connector.base";
import { RequestResponseConnectionAdapter as RequestResponseConnectionAdapterInterface, TransportEvent, TransportMessage } from "../interface/connector.interface";
import { TransmitterConnectionAdapter } from './connector.transmitter';
import { ReceiverConnectionAdapter } from './connector.receiver';
import { filter, map, Observable, Observer, Subject, Subscription, takeWhile } from 'rxjs';

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class RequestResponseConnectionAdapter extends ConnectionAdapter implements RequestResponseConnectionAdapterInterface {
    private transmitterAdapter!: TransmitterConnectionAdapter
    private receiverAdapter!: ReceiverConnectionAdapter

    constructor(transmitterAdapter: TransmitterConnectionAdapter, receiverAdapter: ReceiverConnectionAdapter, adapterId: string) {
        super()
        // logic here
        this.transmitterAdapter = transmitterAdapter
        this.receiverAdapter = receiverAdapter
        this.setAdapterProfile(adapterId)
    }
    setAdapterProfile(adapterId: string): void {
        this.connectorProfile = {
            id: adapterId,
            connectionState: this.connectionStateBus,
            adapter: this.connector
        }    
    }
    
    
    send(message: TransmissionMessage): Observable<TransportMessage> {
        return new Observable((response: Observer<TransportMessage>) => {
            // logic here
            this.transmitterAdapter.emit(message)
            const subscription: Subscription = this.event.pipe(
                filter((message: TransportEvent) => message.event === 'New Message'),
                // take message only specific for this adapter. Although that itself wouldn't be necessary, considerng everything goes through transportEvent. I guess it's for better management
                filter((message:TransportEvent) => (message.data as TransportMessage).target == this.connectorProfile.id), 
                takeWhile((message: TransportEvent) => {
                    const shouldTake = ((message.data as TransportMessage).payload as FisMessage).data !== 'Complete';
                    if (!shouldTake) {
                        response.complete();  // Ensure the observer is completed
                    }
                    return shouldTake;
                }),
                map(message => message.data as TransportMessage)
            ).subscribe((message: TransportMessage) => {
                response.next(message);
            });

            // Clean up on unsubscription
            return () => {
                subscription.unsubscribe();
            };
        })
    }
  

    /* Extended from both transmitter and receiver */
    subscribeMessages(messageFilter: any): Observable<any> {
        throw new Error('Method not implemented.');
    }
    
    emit(message: TransmissionMessage): void {
        throw new Error('Method not implemented.');
    }
    emitStream(message: TransmissionMessage): void {
        throw new Error('Method not implemented.');
    }
}


