import dotenv from 'dotenv';
import { FisAppActor, FisMessage, TransmissionMessage } from "../interface/transport.interface";
import { ConnectionAdapter } from "./connector.base";
import { AdapterProfile, AdaptorTransmissionRole, ConnectionState, TransmitterConnectionAdapter as TransmitterConnectionAdapterInterface, Transport, TransportEvent, TransportMessage, TransportService } from '../interface/connector.interface';
import { BehaviorSubject, Observable, Observer, Subject } from 'rxjs';
import { v4 as uuidv4 } from 'uuid'

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class TransmitterConnectionAdapter extends ConnectionAdapter implements TransmitterConnectionAdapterInterface {
    connectionStateBus: Subject<ConnectionState> = new Subject() 

    constructor(transportService: TransportService, adapterId: string, event: Subject<TransportEvent>, connectionState: Observable<ConnectionState>) {
        super()
        // logic here
        this.event = event
        this.connector = transportService
        this.setAdapterProfile(adapterId)
        connectionState.subscribe(this.connectionStateBus)
    }

    setAdapterProfile(adapterId: string): void {
        this.connectorProfile = {
            id: adapterId,
            connectionState: this.connectionStateBus,
            adapter: this.connector
        }
    }

    emitStream(message: TransmissionMessage): void {
        throw new Error('Method not implemented.');
    }

    emit(message: TransmissionMessage): void {
        (this.connector as TransportService).emit({
            id: uuidv4(),
            dateCreated: new Date(),
            transport: Transport.Websocket,
            target: message.adapterId,
            payload: message.payload
        } as TransportMessage)
    }

    getInfo(): AdapterProfile {
        return this.connectorProfile
    }
    
}


