import { filter, map, Observable, Observer, PartialObserver, Subject, Subscriber, Subscription, takeWhile, Unsubscribable } from 'rxjs';
import { AdapterProfile, AdapterSet, AdaptorTransmissionRole, TransportEvent, TransportMessage } from '../interface/connector.interface';
import { MessageTransmissionBase } from './msg.transmission.base';
import { Bus, EventMessage, FisMessage, MessageReceiver as MessageReceiverInterface, ReceiverProfile, TransmissionMessage, TransmitterProfile } from '../interface/transport.interface'
import { ConnectionAdapter } from '../connector/connector.base';
import { v4 as uuidv4 } from 'uuid'
import { ReceiverConnectionAdapter } from '../connector/connector.receiver';

export class MessageTransmissionReceiver extends MessageTransmissionBase implements MessageReceiverInterface {
    receiverProfile!: ReceiverProfile;
    private incomingTransportMessage: Subject<TransportMessage> = new Subject()

    constructor(profile: ReceiverProfile, role: AdaptorTransmissionRole, event: Observable<TransportEvent>) {
        super();

        this.setReceiver(profile, role)
        this.handleEvent(event)
    }

    setReceiver(receiverProfile: ReceiverProfile, role: AdaptorTransmissionRole): void {
        this.receiverProfile = receiverProfile
        this.transmissionRole = role
    }

    getMessageBus(bus: Bus): Observable<TransmissionMessage> {
        return new Observable((observable: Observer<TransmissionMessage>) => {
            // logic here
            if (bus == Bus.GeneralBus) {
                const subscription: Subscription = this.incomingTransportMessage.pipe(
                    filter((message: TransportMessage) => message.payload === `Incoming Message`),
                    map(message => message.payload as TransportMessage)
                ).subscribe((message: TransportMessage) => {
                    observable.next({
                        adapterId: message.target as string, // assign adapter id so that it knows who to reply back
                        payload: message.payload
                    });
                });

                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
        })
    }

    private handleEvent(event: Observable<TransportEvent>): void {
        event.pipe(
            filter((obj: TransportEvent) => obj.event == `New Adapter`),
            map((obj: TransportEvent) => (obj.data as EventMessage))
        ).subscribe((event: EventMessage) => {
            let adapter = (event.payload as AdapterSet).receiverAdapter
            this.adaptorsArray.push({ id: adapter.getInfo().id, adapter: adapter } as unknown as AdapterProfile)
            adapter.getMessageBus(Bus.GeneralBus).subscribe(this.incomingTransportMessage)
        })
    }
}
