import { MessageTransmissionBase } from "./msg.transmission.base";
import { FisMessage, MessageReceiver, MessageRequestResponse as MessageRequestResponseInterface } from '../interface/transport.interface'
import { filter, Observable, Observer, Subscription, takeWhile } from "rxjs";
import { v4 as uuidv4 } from 'uuid'
import { AdaptorTransmissionRole, RequestResponseConnectionAdapter, TransportEvent } from "../interface/connector.interface";
import { MessageTransmissionReceiver } from "./msg.transmission.receiver";
import { MessageTransmissionTransmitter } from "./msg.transmission.transmitter";

export class MessageTransmissionRequestResponse extends MessageTransmissionBase implements MessageRequestResponseInterface {
    transmitterInstance!: MessageTransmissionTransmitter;
    receiverInstance!: MessageTransmissionReceiver;

    constructor(transmitterInstance: MessageTransmissionTransmitter, receiverInstance: MessageTransmissionReceiver, role: AdaptorTransmissionRole, event: Observable<TransportEvent>) {
        super()
        this.setTransmissionProfile(transmitterInstance, receiverInstance, role)
    }

    setTransmissionProfile(transmitterInstance: MessageTransmissionTransmitter, receiverInstance: MessageTransmissionReceiver, role: AdaptorTransmissionRole): void {
        this.transmitterInstance = transmitterInstance
        this.receiverInstance = receiverInstance
        this.transmissionRole = role
    }

    // To be Enhanced. This is actually wrong at the moment
    send(message: FisMessage): Observable<FisMessage> {
        return new Observable((response: Observer<FisMessage>) => {
            this.transmitterInstance.send({
                transmitter: '',
                payload: message
            });

            const subscription: Subscription = this.incomingMessageBus
                .pipe(
                    // Filter for matching message IDs
                    filter(incomingMessage => incomingMessage.header.messageID === message.header.messageID),
                    // Complete the observable when the data is 'Complete'
                    takeWhile(incomingMessage => incomingMessage.data !== 'Complete')
                )
                .subscribe({
                    next: incomingMessage => {
                        response.next(incomingMessage);
                        // Complete the observable if data is 'Complete'
                        if (incomingMessage.data === 'Complete') {
                            response.complete();
                            subscription.unsubscribe();
                        }
                    }
                });
        });
    }
}